#!/bin/bash
####################################################################################################################
# Novell Inc.
# 1800 South Novell Place
# Provo, UT 84606-6194
# Script Name:	    objectSids_menu.sh
# Description:	    This will generate an ldif to remove the objectSid from a user.  
#                   It will prompt for a user name.
#                   Enter the fully distinguished name at the prompt.
#	                              
# %Version:	    2.0
# %Creation Date:   Tues Oct 18 15:27:11 MST 2011  
# %Created by:	    Rance Burker - Novell Technical Services
# %Modification:    modified example, must use domain name
# %Modified on:     Thu Aug 29 12:20:03 MDT 2013
# %Modified by:     Rance Burker
# %Change log:      Added backup/restore and ability to remove objectSids from a list 

####################################################################################################################

### Variables ###
SCRIPTNAME=$(basename $0)
VERSION=2.0
SERVERNAME=`/bin/hostname`
DAEMON_PID=""
PWD=`/bin/pwd` >/dev/null
LOGDIR=/root
LOCATION=/tmp

#Colors
RED='\e[1;91m'
GREEN='\e[1;92m'
URED='\e[4;91m'
UGREEN='\e[4;92m'
BOLD='\e[1m'
UBOLD='\e[4m'
NC='\e[0m'


export LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf 
defaultnamingcontext=`/usr/bin/ldapsearch -x -b "" -s base defaultnamingcontext | grep -i 'defaultnamingcontext: ' | awk '{print $2}'`


backupObjectSids(){
    if [ -d /var/opt/novell/xad/backup ]; then
        &> /dev/null
    else
        /bin/mkdir /var/opt/novell/xad/backup
    fi

    OBJECTSIDBK=/var/opt/novell/xad/backup/objectSids_backedUpOn_`date -I`.ldif
    /usr/bin/ldapsearch -Y EXTERNAL -LLL -Q -b "$defaultnamingcontext" -s sub "(&(objectclass=*)(objectSid=*))" dn objectSid|sed s[objectSid['changetype:modify\nreplace:objectSid\nobjectSid'[g | grep -v ^# |sed -e :a -e '$!N;s/\n //;ta' -e 'P;D' > $OBJECTSIDBK

    UIDNUMBERBK=/var/opt/novell/xad/backup/uidNumbers_backedUpOn_`date -I`.ldif
    /usr/bin/ldapsearch -Y EXTERNAL -LLL -Q -b "$defaultnamingcontext" -s sub "(&(objectclass=*)(uidNumber=*))" dn uidNumber|sed s[uidNumber['changetype:modify\nreplace:uidNumber\nuidNumber'[g | grep -v ^# |sed -e :a -e '$!N;s/\n //;ta' -e 'P;D' > $UIDNUMBERBK

    echo
    echo -e "The objectSids backup is located at $OBJECTSIDBK\n"
}

restoreObjectSids(){
    echo -ne "Enter the path and file to restore: "
    read RESTOREFILE \n
    echo -e "The file that will be restored is $RESTOREFILE"
    echo -ne "Is this correct (y/n): "
    read REPLY
    if [[ $REPLY =~ ^[Yy]$ ]]; then 
        /usr/bin/ldapmodify -Y EXTERNAL -Q -f $RESTOREFILE
        echo -e "Restored $RESTOREFILE"
    else
        exit 1;
    fi
}

removeSidEnter(){
    echo -n "Enter object name (ex. cn=user,dc=dsfw,dc=lan): "
    read DELOBJECT
    if [[ "$DELOBJECT" = ""  ]]; then DELOBJECT="All Users and Computers"; fi
    echo -e "Creating ldif to remove objectSid for $DELOBJECT\n"
    if [[ "$DELOBJECT" = "All Users and Computers"  ]]; then
        echo "WARNING: you are about to remove the objectSid from all users excluding the Builtin, Configuration, and Users Constainers"
        echo -ne "Do you want to continue (y/n):"
        TIMELIMIT=20
        read -t $TIMELIMIT REPLY # set timelimit on REPLY
        if [ -z "$REPLY" ]; then   # if REPLY is null then exit
            echo ""
            echo -e "    Timeout, did not remove objectSids "
        elif [[ $REPLY =~ ^[Yy]$ ]]; then # if yes then Add
    /usr/bin/ldapsearch -Y EXTERNAL -LLL -Q -b "$DELOBJECT" "(&(objectclass=user)(!(|(cn:dn:=Builtin)(cn:dn:=Configuration)(cn:dn:=Users)))(objectSid=*))" dn | sed -e :a -e '$!N;s/\n //;ta' -e 'P;D'|sed -e'/^dn/ a\changetype: modify' |sed -e'/^changetype:/ a\delete: objectSid' >$LOCATION/delete_objectSid.ldif
        echo -e " The following objectSids will be removed"
        cat $LOCATION/delete_objectSid.ldif
        echo
        echo -ne " Are you sure you want to remove the objectSid from all users (y/n):"
        read -t $TIMELIMIT REPLY # set timelimit on REPLY
        if [ -z "$REPLY" ]; then   # if REPLY is null then exit
            echo ""
            echo -e "    Timeout, did not remove objectSids "
        elif [[ $REPLY =~ ^[Yy]$ ]]; then # if yes then Add
            backupObjectSids
            /usr/bin/ldapmodify -Y EXTERNAL -Q -f $LOCATION/delete_objectSid.ldif
        fi;
        fi;
    else
    sleep 2;
    /usr/bin/ldapsearch -Y EXTERNAL -LLL -Q -b "$DELOBJECT" "(&(objectclass=user)(!(|(cn:dn:=Builtin)(cn:dn:=Configuration)(cn:dn:=Users)))(objectSid=*))" dn | sed -e :a -e '$!N;s/\n //;ta' -e 'P;D'|sed -e'/^dn/ a\changetype: modify' |sed -e'/^changetype:/ a\delete: objectSid' >$LOCATION/delete_objectSid.ldif
#    ldapsearch -Y EXTERNAL -LLL -Q -b "$DELOBJECT" "(&(objectclass=user)(objectSid=*))" dn | sed -e :a -e '$!N;s/\n //;ta' -e 'P;D'|sed -e'/^dn/ a\changetype: modify' |sed -e'/^changetype:/ a\delete: objectSid' >$LOCATION/delete_objectSid.ldif

    /usr/bin/ldapmodify -Y EXTERNAL -Q -f $LOCATION/delete_objectSid.ldif

    rm $LOCATION/delete_objectSid.ldif
    fi;
}

removeSidPrompt(){
    echo
    echo -e "If this script is ran from the server or a workstation running X Windows (like xming) a second terminal window will be displayed listing the objects, objectSids, and createTimestamps (Creation Time).\n"
    echo -e "Compare the objectSids (usually the last 10 characters) to see which objects have duplicate objectSids.\n"
    echo -e "Generally the object with the oldest creation timeStamp is the object with the original objectSid value and the newest createTimestamp is the object with the duplicate value.\n"
    echo -e "Not always, but a good pratice is to delete the objectSid from the object with the newest createTimestamp.\n"
    echo -ne "Do you want to continue (y/n):"
        TIMELIMIT=90
        read -t $TIMELIMIT REPLY # set timelimit on REPLY
        if [ -z "$REPLY" ]; then   # if REPLY is null then exit
            echo ""
            echo -e "    Timeout, did not remove objectSids from list of objects"
        elif [[ $REPLY =~ ^[Yy]$ ]]; then # if yes then Add

            DUPLICATE_SIDS=/tmp/duplicateObjectsids2.log
            /usr/bin/ldapsearch -Y EXTERNAL -LLL -Q -b "$defaultnamingcontext" -s sub '(&(objectclass=user)(!(|(cn:dn:=Builtin)(cn:dn:=Configuration)(cn:dn:=Users)))(objectSid=*))' cn objectsid createTimestamp >/tmp/objectSids.log

            sids=$(grep -i objectsid:: /tmp/objectSids.log|sort |uniq -c |sort -n |grep -v ' 1 ' |awk '{print $3}')
            sids2=$(echo $sids | sed -e 's/ /\\|/g')
            grep -e $sids2 -B2 -A3 /tmp/objectSids.log  >/tmp/duplicateObjectsids.log
            grep -e dn: /tmp/duplicateObjectsids.log | awk -F "dn: " '{print $2}'> $DUPLICATE_SIDS
            sed -i -e '/^$/d' $DUPLICATE_SIDS > /dev/null
            if [ -s "$DUPLICATE_SIDS" ]; then
                xterm -hold -e less /tmp/duplicateObjectsids.log & sleep 5
                echo
                echo -e "Objects with Duplicate ObjectSids (rids)\n"
                cat $DUPLICATE_SIDS
                echo "Open another terminal and less the /tmp/duplicateObjectsids.log for more info"
                grep -e dn: /tmp/duplicateObjectsids.log | awk -F "dn: " '{print $2}'> $DUPLICATE_SIDS

                ctr0=0;while read i ; do echo $i;list[$ctr0]="${i}"; let ctr0=ctr0+1; done < ${DUPLICATE_SIDS} > /dev/null

                for i in `seq 0 "${#list[@]}"`; do objsid="${list[$i]}"; if [[ -n ${objsid} ]]; then currentObject="${objsid}"; else break; fi;
                    echo 
                    echo -ne "Do you want to remove the objectSids from $currentObject (y/n):"
                    TIMELIMIT=60
                    read -t $TIMELIMIT REPLY # set timelimit on REPLY
                    if [ -z "$REPLY" ]; then   # if REPLY is null then exit
                        echo ""
                        echo -e "    Timeout, did not remove objectSids from list of objects"
                    elif [[ $REPLY =~ ^[Yy]$ ]]; then # if yes then Add
                        /usr/bin/ldapsearch -Y EXTERNAL -LLL -Q -b "$currentObject" -s base dn | sed -e :a -e '$!N;s/\n //;ta' -e 'P;D' |sed -e'/^dn/ a\changetype: modify\ndelete: objectSid'>/tmp/delete_objectSid.ldif
                        echo "Removing $currentObject from list"
                        /usr/bin/ldapmodify -Y EXTERNAL -Q -f /tmp/delete_objectSid.ldif > /dev/null
                        echo "Finished removing objectSid from $currentObject"
                    fi;
                done;
            else echo -e "There are no objects with duplicate objectSids\n"; fi;
    fi;
}

main(){
    if ["$1" = ""]; then
    clear
    echo
    echo -e "   ${BOLD}Script Name: $SCRIPTNAME $VERSION${NC}"
    echo -e "   This script can backup, restore, or delete objectSids"
    echo
    echo -e "   Press ${BOLD}1${NC} to Backup objectSids"
    echo -e "   Press ${BOLD}2${NC} to Restore objectSids from backup"
    echo -e "   Press ${BOLD}3${NC} to Delete objectSid manually by entering an object at the prompt"
    echo -e "   Press ${BOLD}4${NC} to Search and list duplicate objectSids and evaluate one by one the objectSid to delete"
    echo
    echo -e "   Press ${BOLD}0${NC} or Press ENTER to Exit"
    echo
    echo -n "   Enter a number:  "
    read IN
    else
    IN="$1"
    fi
case $IN in
1) 
backupObjectSids
;;
2)
restoreObjectSids
;;
3)
removeSidEnter
;;
4)
removeSidPrompt
;;
*) exit
;;
esac
exit 0
}


# Run the main funtion
main
exit 0
